@@ -7,11 +7,11 @@
 setClass("lavData",
     representation(
         data.type="character",     # "full", "moment" or "none"
-        ngroups="integer",         # number of groups
         group="character",         # group variable
-        nlevels="integer",         # number of levels
-        cluster="character",       # cluster variable(s)
+        ngroups="integer",         # number of groups
         group.label="character",   # group labels
+        cluster="character",       # cluster variable(s)
+        nlevels="integer",         # number of levels
         level.label="character",   # level labels
         std.ov="logical",          # standardize observed variables?
         nobs="list",               # effective number of observations
@@ -59,7 +59,7 @@ setClass("lavSampleStats",         # sample moments
         group.w="list",            # group weight
 
         nobs="list",               # effective number of obs (per group)
-        ntotal="integer",          # total number of obs (all groups)
+        ntotal="numeric",          # total number of obs (all groups)
         ngroups="integer",         # number of groups
         x.idx="list",              # x.idx if fixed.x = TRUE
 
@@ -155,10 +155,15 @@ setClass("lavModel",          # MATRIX representation of the sem model
 
         ov.efa.idx="list",
         lv.efa.idx="list",
+
+        rv.ov="list",
+        rv.lv="list",
+
         H="list",
         lv.order="list",
 
-        estimator="character"
+        estimator="character",
+        estimator.args="list"
     )
 )
 
@@ -232,6 +237,7 @@ setClass("lavaanList",
         impliedList     = "list",
         h1List          = "list",
         loglikList      = "list",
+        baselineList    = "list",
         funList         = "list",
         external        = "list"       # optional slot, for add-on packages
     )
