@@ -26,6 +26,11 @@ lav_model_information <- function(lavmodel       = NULL,
     information <- lavoptions$information[1] # ALWAYS used the first one
                                              # called can control it
 
+    # rotation?
+    #if(!is.null(lavoptions$rotation) && lavoptions$rotation != "none") {
+    #    use.ginv <- TRUE
+    #}
+
     if(is.null(lavh1)) {
         lavh1 <- lav_h1_implied_logl(lavdata = lavdata,
                                      lavsamplestats = lavsamplestats,
@@ -479,9 +484,12 @@ lav_model_information_augment_invert <- function(lavmodel    = NULL,
             H10 <- matrix(0, ncol(information), nrow(H))
             DL <- 2*diag(lambda, nrow(H), nrow(H))
             # FIXME: better include inactive + slacks??
-            E3 <- rbind( cbind(     information,  H10, t(H)),
-                         cbind(          t(H10),   DL,  H0),
-                         cbind(               H,   H0,  H0)  )
+            #INFO <- information
+            # or
+            INFO <- information + crossprod(H)
+            E3 <- rbind( cbind(   INFO,  H10, t(H)),
+                         cbind( t(H10),   DL,  H0),
+                         cbind(      H,   H0,  H0)  )
             information <- E3
         }
     }
