@@ -5,14 +5,15 @@
 
 # take model-implied mean+variance matrices, and reorder/augment them
 # to facilitate computing of (log)likelihood in the two-level case
-#
+
+# when conditional.x = FALSE:
 # - sigma.w and sigma.b: same dimensions, level-1 variables only
 # - sigma.zz: level-2 variables only
 # - sigma.yz: cov(level-1, level-2)
 # - mu.y: level-1 variables only (mu.w + mu.b)
 # - mu.w: y within  part
 # - mu.b: y between part
-# - my.z: level-2 variables only
+# - mu.z: level-2 variables only
 lav_mvnorm_cluster_implied22l <- function(Lp           = NULL,
                                           implied      = NULL,
                                           Mu.W         = NULL,
@@ -116,11 +117,15 @@ lav_mvnorm_cluster_2l2implied <- function(Lp,
         mu.b <- mu.y
         mu.w.tilde <- numeric( p.tilde )
         mu.w.tilde[ ov.idx[[1]] ] <- mu.y
-        if(length(within.idx) > 0L) {
-            mu.w.tilde[  -within.idx ] <- 0
-        } else {
-            mu.w.tilde[] <- 0
-        }
+
+        # NO NEED TO SET THIS TO ZERO!
+        # otherwise, we get non-symmetric Hessian!! 0.6-5
+
+        #if(length(within.idx) > 0L) {
+        #    mu.w.tilde[  -within.idx ] <- 0
+        #} else {
+        #    mu.w.tilde[] <- 0
+        #}
         mu.w <- mu.w.tilde[ ov.idx[[1]] ]
     }
 
@@ -165,7 +170,6 @@ lav_mvnorm_cluster_loglik_samplestats_2l <- function(YLp          = NULL,
                                                      Mu.B         = NULL,
                                                      Sigma.B      = NULL,
                                                      Sinv.method  = "eigen",
-                                                     x.idx        = integer(0L),
                                                      log2pi       = FALSE,
                                                      minus.two    = TRUE) {
 
@@ -176,18 +180,6 @@ lav_mvnorm_cluster_loglik_samplestats_2l <- function(YLp          = NULL,
     sigma.w <- out$sigma.w; sigma.b <- out$sigma.b
     sigma.zz <- out$sigma.zz; sigma.yz <- out$sigma.yz
 
-    #if(length(x.idx) > 0L) {
-    #    x1.idx <- Lp$ov.idx[[1]][ which( Lp$ov.idx[[1]] %in% x.idx )]
-    #    x2.idx <- Lp$ov.idx[[2]][ which( Lp$ov.idx[[2]] %in% x.idx )]
-    #    if(length(x1.idx) > 0L) {
-    #        mu.y.x <- mu.y[x1.idx]
-    #        sigma.w.x <- sigma.w[x1.idx, x1.idx, drop = FALSE]
-    #        sigma.b.x <- sigma.b[x1.idx, x1.idx, drop = FALSE]
-    #    }
-    #    if(length(x2.idx) > 0L) {
-    #    }
-    #}
-
     # Lp
     nclusters       <- Lp$nclusters[[2]]
     cluster.size    <- Lp$cluster.size[[2]]
@@ -196,22 +188,14 @@ lav_mvnorm_cluster_loglik_samplestats_2l <- function(YLp          = NULL,
     ncluster.sizes  <- Lp$ncluster.sizes[[2]]
     cluster.size.ns <- Lp$cluster.size.ns[[2]]
 
-    # Y1
+    # Y1 samplestats
     if(length(between.idx) > 0L) {
         S.PW <- YLp[[2]]$Sigma.W[-between.idx, -between.idx, drop = FALSE]
-        #if(length(x.idx) > 0L) {
-        #    S.PW.x <- S.PW[x.idx, x.idx, drop = FALSE]
-        #}
     } else {
         S.PW <- YLp[[2]]$Sigma.W
-        #if(length(x.idx) > 0L) {
-        #    S.PW.x <- S.PW[x.idx, x.idx, drop = FALSE]
-        #}
     }
 
-    # Y2
-    #Y2 <- YLp[[2]]$Y2
-    #Y2.cm <- t( t(Y2) - mu.b )
+    # Y2 samplestats
     cov.d <- YLp[[2]]$cov.d
     mean.d <- YLp[[2]]$mean.d
 
@@ -245,7 +229,7 @@ lav_mvnorm_cluster_loglik_samplestats_2l <- function(YLp          = NULL,
         # data between
         Y2Yc <- (cov.d[[clz]] + tcrossprod(mean.d[[clz]] - c(mu.z, mu.y)))
 
-        # FIXME: avoid reorder/b.idx, so can use between.idx
+        # FIXME: avoid reorder/b.idx, so we can use between.idx
         if(length(between.idx) > 0L) {
             b.idx <- seq_len(length(Lp$between.idx[[2]]))
             Y2Yc.zz <- Y2Yc[ b.idx, b.idx, drop = FALSE]
@@ -262,7 +246,7 @@ lav_mvnorm_cluster_loglik_samplestats_2l <- function(YLp          = NULL,
         sigma.j.logdet <- attr(sigma.j.inv, "logdet")
         attr(sigma.j.inv, "logdet") <- NULL
 
-        # check: what if sigma.j is non-pd?
+        # check: what if sigma.j is non-pd? should not happen
         if(is.na(sigma.j.logdet)) {
             # stop, and return NA right away
             #return(as.numeric(NA))
@@ -296,7 +280,7 @@ lav_mvnorm_cluster_loglik_samplestats_2l <- function(YLp          = NULL,
         B[clz] <- q.zz + 2*q.yz - q.yyc
     }
     # q.yya + q.yyb
-    # there reason why we multiply the trace by 'N - nclusters' is
+    # the reason why we multiply the trace by 'N - nclusters' is
     # S.PW has been divided by 'N - nclusters'
     q.W <- sum(cluster.size - 1) * sum(sigma.w.inv * S.PW)
     # logdet within part
@@ -321,6 +305,11 @@ lav_mvnorm_cluster_loglik_samplestats_2l <- function(YLp          = NULL,
         loglik <- loglik + constant
     }
 
+    # loglik.x (only if loglik is requested)
+    if(length(unlist(Lp$ov.x.idx)) > 0L && log2pi && !minus.two) {
+        loglik <- loglik - YLp[[2]]$loglik.x
+    }
+
     loglik
 }
 
@@ -765,7 +754,19 @@ lav_mvnorm_cluster_scores_2l <- function(Y1           = NULL,
                                              drop = FALSE ] )
         Sigma.B <- Sigma.B.tilde[ , col.idx, drop = FALSE ]
     } else {
-        Sigma.B <- G.Sigma.b
+        p.tilde.star <- p.tilde*(p.tilde+1)/2
+        B.tilde <- lav_matrix_vech_reverse(seq_len(p.tilde.star))
+
+        Sigma.B.tilde <- matrix(0, nclusters, p.tilde.star)
+
+        col.idx <- lav_matrix_vech( B.tilde[ ov.idx[[1]], ov.idx[[1]],
+                                             drop = FALSE ] )
+        Sigma.B.tilde[ , col.idx ] <- G.Sigma.b
+
+        col.idx <- lav_matrix_vech( B.tilde[ ov.idx[[2]], ov.idx[[2]],
+                                             drop = FALSE ] )
+        Sigma.B <- Sigma.B.tilde[ , col.idx, drop = FALSE ]
+        #Sigma.B <- G.Sigma.b
     }
 
     SCORES <- cbind(Mu.W, Sigma.W, Mu.B, Sigma.B)
@@ -997,6 +998,14 @@ lav_mvnorm_cluster_information_expected_delta <- function(Lp     = NULL,
     Delta.W.tilde.Mu[ov.idx[[1]],] <- Delta.W[1:nw,]
     Delta.B.tilde.Mu[ov.idx[[2]],] <- Delta.B[1:nb,]
 
+    # correct Delta to reflect Mu.W[ both.idx ] is added to Mu.B[ both.idx ]
+    # changed in 0.6-5
+    Delta.B.tilde.Mu[ Lp$both.idx[[2]], ] <-
+        ( Delta.B.tilde.Mu[ Lp$both.idx[[2]], ] +
+          Delta.W.tilde.Mu[ Lp$both.idx[[2]], ] )
+    Delta.W.tilde.Mu[Lp$both.idx[[2]], ] <- 0
+
+
     B.tilde <- lav_matrix_vech_reverse(seq_len(p.tilde.star))
     w.idx <- lav_matrix_vech( B.tilde[ ov.idx[[1]], ov.idx[[1]], drop = FALSE] )
     b.idx <- lav_matrix_vech( B.tilde[ ov.idx[[2]], ov.idx[[2]], drop = FALSE] )
@@ -1047,16 +1056,24 @@ lav_mvnorm_cluster_information_expected_delta <- function(Lp     = NULL,
         information.j <- information.j + n.s[clz]*info.j
     }
 
-    Sigma.W.inv <- lav_matrix_symmetric_inverse(S = Sigma.W, logdet = FALSE,
+
+    Sigma.W.inv <- lav_matrix_symmetric_inverse(S = sigma.w, logdet = FALSE,
                                                 Sinv.method = Sinv.method)
     I11.w <- Sigma.W.inv
     I22.w <- 0.5 * lav_matrix_duplication_pre_post(Sigma.W.inv %x% Sigma.W.inv)
     I.w <- lav_matrix_bdiag(I11.w, I22.w)
+
+    # force zero for means both.idx in within part
+    # changed in 0.6-5
+    I.w[Lp$both.idx[[2]],] <- 0
+    I.w[,Lp$both.idx[[2]]] <- 0
+
     information.w <- (nobs - nclusters) * ( t(Delta.W) %*% I.w %*% Delta.W )
 
     # unit information
     information <- 1/Lp$nclusters[[2]] * (information.w + information.j)
 
+
     information
 }
 
@@ -1212,7 +1229,8 @@ lav_mvnorm_cluster_em_sat <- function(YLp            = NULL,
     if(length(between.idx) > 0L) {
         Z <- Y2[, between.idx, drop = FALSE]
         mu.z <- colMeans(Y2)[between.idx]
-        sigma.zz <- 1/Lp$nclusters[[2]] * crossprod(Z) - tcrossprod(mu.z)
+        sigma.zz <- cov(Z) * (Lp$nclusters[[2]] - 1L)/Lp$nclusters[[2]]
+        #sigma.zz <- 1/Lp$nclusters[[2]] * crossprod(Z) - tcrossprod(mu.z)
         #Y1Y1 <- Y1Y1[-between.idx, -between.idx, drop=FALSE]
     }
 
@@ -1346,7 +1364,8 @@ lav_mvnorm_cluster_em_h0 <- function(lavsamplestats = NULL,
     if(length(between.idx) > 0L) {
         Z <- Y2[, between.idx, drop = FALSE]
         mu.z <- colMeans(Y2)[between.idx]
-        sigma.zz <- 1/Lp$nclusters[[2]] * crossprod(Z) - tcrossprod(mu.z)
+        sigma.zz <- cov(Z) * (Lp$nclusters[[2]] - 1L)/Lp$nclusters[[2]]
+        #sigma.zz <- 1/Lp$nclusters[[2]] * crossprod(Z) - tcrossprod(mu.z)
         #Y1Y1 <- Y1Y1[-between.idx, -between.idx, drop=FALSE]
     }
 
@@ -1380,6 +1399,8 @@ lav_mvnorm_cluster_em_h0 <- function(lavsamplestats = NULL,
 
         # fit two-group model
         local.partable <- lavpartable
+        # if a group column exists, delete it (it will be overriden anyway)
+        local.partable$group <- NULL
         level.idx <- which(names(local.partable) == "level")
         names(local.partable)[level.idx] <- "group"
         local.partable$est <- NULL
@@ -1405,6 +1426,8 @@ lav_mvnorm_cluster_em_h0 <- function(lavsamplestats = NULL,
                             check.post = FALSE,
                             check.gradient = FALSE,
                             check.vcov = FALSE,
+                            baseline = FALSE,
+                            h1 = FALSE,
                             se = "none",
                             test = "none")
 
@@ -1472,8 +1495,13 @@ lav_mvnorm_cluster_em_h0 <- function(lavsamplestats = NULL,
     # add attributes
     if(i < max.iter) {
         attr(x, "converged") <- TRUE
+        attr(x, "warn.txt")  <- ""
     } else {
         attr(x, "converged") <- FALSE
+        attr(x, "warn.txt")  <- paste("maxmimum number of iterations (",
+                                      max.iter, ") ",
+                                      "was reached without convergence.\n",
+                                      sep = "")
     }
     attr(x, "iterations") <- i
     attr(x, "control") <- list(em.iter.max = max.iter,
