@@ -8,14 +8,78 @@
 #    note that for ML (and ULS and DWLS), the 'estimates' of the
 #    independence model are simply the observed variances
 #    but for GLS and WLS, this is not the case!!
-lav_object_independence <- function(object, se = FALSE, verbose = FALSE,
-                                    warn = FALSE) {
+lav_object_independence <- function(object         = NULL,
+                                    # or
+                                    lavsamplestats = NULL,
+                                    lavdata        = NULL,
+                                    lavcache       = NULL,
+                                    lavoptions     = NULL,
+                                    lavpta         = NULL,
+                                    lavh1          = NULL,
+                                    # local options
+                                    se             = FALSE,
+                                    verbose        = FALSE,
+                                    warn           = FALSE) {
+
+     # object or slots?
+     if(!is.null(object)) {
+        stopifnot(inherits(object, "lavaan"))
+
+        # extract needed slots
+        lavsamplestats <- object@SampleStats
+        lavdata <- object@Data
+        lavcache <- object@Cache
+        lavoptions <- object@Options
+        lavpta     <- object@pta
+        if(.hasSlot(object, "h1"))  {
+            lavh1 <- object@h1
+        } else {
+            lavh1 <- lav_h1_logl(lavdata = object@Data,
+                                 lavsamplestats = object@SampleStats,
+                                 lavoptions = object@Options)
+        }
+        if(is.null(lavoptions$estimator.args)) {
+            lavoptions$estimator.args <- list()
+        }
+    }
 
     # construct parameter table for independence model
-    lavpartable <- lav_partable_independence(object)
+    lavpartable <- lav_partable_indep_or_unrestricted(lavobject = NULL,
+        lavdata = lavdata, lavpta = lavpta, lavoptions = lavoptions,
+        lavsamplestats = lavsamplestats, lavh1 = lavh1, independent = TRUE)
 
-    # adapt options
-    lavoptions <- object@Options
+    # new in 0.6-6: add lower bounds for ov.var
+    if(!is.null(lavoptions$optim.bounds)) {
+        lavoptions$bounds <- "doe.maar"
+        lavoptions$optim.bounds <- list(lower = "ov.var")
+        lavpartable <- lav_partable_add_bounds(partable = lavpartable,
+            lavpta = lavpta, lavh1 = lavh1, lavdata = lavdata,
+            lavsamplestats = lavsamplestats, lavoptions = lavoptions)
+    }
+
+    # new in 0.6-8: if DLS, change to sample-based
+    if(lavoptions$estimator == "DLS") {
+        if(lavoptions$estimator.args$dls.GammaNT == "sample") {
+            # nothing to do
+        } else {
+           lavoptions$estimator.args$dls.GammaNT <- "sample"
+           dls.a <- lavoptions$estimator.args$dls.a
+           for(g in 1:lavsamplestats@ngroups) {
+               GammaNT <- lav_samplestats_Gamma_NT(
+                        COV            = lavsamplestats@cov[[g]],
+                        MEAN           = lavsamplestats@mean[[g]],
+                        rescale        = FALSE,
+                        x.idx          = lavsamplestats@x.idx[[g]],
+                        fixed.x        = lavoptions$fixed.x,
+                        conditional.x  = lavoptions$conditional.x,
+                        meanstructure  = lavoptions$meanstructure,
+                        slopestructure = lavoptions$conditional.x)
+                W.DLS <- (1 - dls.a)*lavsamplestats@NACOV[[g]] + dls.a*GammaNT
+                # overwrite
+                lavsamplestats@WLS.V[[g]] <- lav_matrix_symmetric_inverse(W.DLS)
+            }
+        }
+    }
 
     # se
     if(se) {
@@ -27,15 +91,16 @@ lav_object_independence <- function(object, se = FALSE, verbose = FALSE,
         lavoptions$se <- "none"
     }
 
-    # set baseline/h1 to FALSE
-    lavoptions$h1 <- FALSE
-    lavoptions$baseline <- FALSE
-    lavoptions$loglik <- TRUE # eg for multilevel
-    lavoptions$implied <- TRUE #, needed for loglik
+    # change options
+    lavoptions$h1 <- FALSE          # already provided by lavh1
+    lavoptions$baseline <- FALSE    # of course
+    lavoptions$loglik <- TRUE       # eg for multilevel
+    lavoptions$implied <- TRUE      # needed for loglik (multilevel)
     lavoptions$check.start <- FALSE
     lavoptions$check.gradient <- FALSE
     lavoptions$check.post <- FALSE
     lavoptions$check.vcov <- FALSE
+    lavoptions$optim.bounds <- list() # we already have the bounds
 
     # ALWAYS do.fit
     lavoptions$do.fit  <- TRUE
@@ -58,19 +123,11 @@ lav_object_independence <- function(object, se = FALSE, verbose = FALSE,
     # lavpartable, and they should be in the right order (unlike the
     # intercepts)
 
-    if(.hasSlot(object, "h1"))  {
-        lavh1 <- object@h1
-    } else {
-        lavh1 <- lav_h1_logl(lavdata = object@Data,
-                             lavsamplestats = object@SampleStats,
-                             lavoptions = object@Options)
-    }
-
     FIT <- lavaan(lavpartable,
                   slotOptions     = lavoptions,
-                  slotSampleStats = object@SampleStats,
-                  slotData        = object@Data,
-                  slotCache       = object@Cache,
+                  slotSampleStats = lavsamplestats,
+                  slotData        = lavdata,
+                  slotCache       = lavcache,
                   sloth1          = lavh1)
 
     FIT
@@ -283,10 +340,12 @@ lav_object_extended <- function(object, add = NULL,
         lavoptions$implied <- FALSE
         lavoptions$baseline <- FALSE
         lavoptions$loglik <- FALSE
+        lavoptions$estimator.args <- list()
 
         # add a few slots
         object@Data@weights <- vector("list", object@Data@ngroups)
         object@Model@estimator <- object@Options$estimator
+        object@Model@estimator.args <- list()
 
         lavh1 <- lav_h1_logl(lavdata = object@Data,
                              lavsamplestats = object@SampleStats,
