@@ -1,4 +1,4 @@
-# constructor for the ltavParTable model description
+# lavaan parameter table
 #
 # initial version: YR 22/05/2009
 #  major revision: YR 02/11/2010: - FLATTEN the model syntax and turn it into a
@@ -13,6 +13,7 @@
 #                move syntax-based code to lav_syntax.R
 # - 26 April 2016: handle multiple 'blocks' (levels, classes, groups, ...)
 # - 24 March 2019: handle efa sets
+# - 23 May   2020: support for random slopes
 
 lavaanify <- lavParTable <- function(
 
@@ -27,7 +28,7 @@ lavaanify <- lavParTable <- function(
                       std.lv           = FALSE,
                       effect.coding    = "",
                       conditional.x    = FALSE,
-                      fixed.x          = TRUE,
+                      fixed.x          = FALSE,
                       parameterization = "delta",
                       constraints      = NULL,
 
@@ -132,7 +133,9 @@ lavaanify <- lavParTable <- function(
     }
 
     # check for meanstructure
-    if(any(FLAT$op == "~1")) meanstructure <- TRUE
+    if(any(FLAT$op == "~1")) {
+        meanstructure <- TRUE
+    }
 
     # check for block identifiers in the syntax (op = ":")
     n.block.flat <- length(which(FLAT$op == ":"))
@@ -198,6 +201,51 @@ lavaanify <- lavParTable <- function(
             FLAT.block <- FLAT[(BLOCK.op.idx[block]+1L):(BLOCK.op.idx[block+1]-1L),]
             # rm 'block' column (if any) in FLAT.block
             FLAT.block$block <- NULL
+
+            # new in 0.6-7: check for random slopes, add them here
+            if(block.lhs == "level" &&
+               block > 1L && # FIXME: multigroup, multilevel
+               !is.null(FLAT$rv) &&
+               any(nchar(FLAT$rv) > 0L)) {
+                lv.names.rv <- unique(FLAT$rv[nchar(FLAT$rv) > 0L])
+                for(i in 1:length(lv.names.rv)) {
+                    # add phantom latent variable
+                    TMP <- FLAT.block[1,]
+                    TMP$lhs <- lv.names.rv[i]; TMP$op <- "=~"
+                    TMP$rhs     <- lv.names.rv[i]
+                    TMP$mod.idx <- max(FLAT$mod.idx) + i
+                    TMP$fixed   <- "0"
+                    TMP$start   <- ""; TMP$lower   <- ""; TMP$upper   <- ""
+                    TMP$label   <- ""; TMP$prior   <- ""; TMP$efa     <- ""
+                    TMP$rv      <- lv.names.rv[i]
+                    FLAT.block <- rbind(FLAT.block, TMP, deparse.level = 0L)
+                    MOD <- c(MOD, list(list(fixed = 0)))
+                }
+            }
+
+            # new in 0.6-8: if multilevel, use 'global' ov.names.x
+            if(fixed.x && block.lhs == "level") {
+                OV.NAMES.X <- lav_partable_vnames(FLAT, "ov.x") # global
+                ov.names.x.block <- lav_partable_vnames(FLAT.block, "ov.x")
+                if(length(ov.names.x.block) > 0L) {
+                    idx <- which(!ov.names.x.block %in% OV.NAMES.X)
+                    if(length(idx) > 0L) {
+                        # warn!
+                        txt <- c("the variable(s) [",
+                         paste0(ov.names.x.block[idx], collapse = " "), "] ",
+						 "are exogenous at one level, but endogenous at ",
+                         "another level. These variables will be treated as ",
+                         "endogenous, and their variances/intercepts will be ",
+                         "freely estimated. To remove this warning, use ",
+                         "fixed.x = FALSE.")
+                        warning(lav_txt2message(txt))
+                        ov.names.x.block <- ov.names.x.block[-idx]
+                    }
+                }
+            } else {
+                ov.names.x.block <- NULL
+            }
+
             LIST.block <- lav_partable_flat(FLAT.block, blocks = BLOCK.lhs,
                 block.id = block.id,
                 meanstructure = meanstructure,
@@ -213,7 +261,8 @@ lavaanify <- lavParTable <- function(
                 auto.cov.y = auto.cov.y, auto.th = auto.th,
                 auto.delta = auto.delta, auto.efa = auto.efa,
                 varTable = varTable, group.equal = NULL,
-                group.w.free = group.w.free, ngroups = 1L)
+                group.w.free = group.w.free, ngroups = 1L,
+                ov.names.x.block = ov.names.x.block)
             LIST.block <- as.data.frame(LIST.block, stringsAsFactors = FALSE)
 
             # add block columns with current values in BLOCK.rhs
@@ -304,8 +353,8 @@ lavaanify <- lavParTable <- function(
        LIST$ustart[scale.idx] <- 1
     }
 
-
     # apply user-specified modifiers
+    warn.about.single.label <- FALSE
     if(length(MOD)) {
         for(el in 1:length(MOD)) {
             idx <- which(LIST$mod.idx == el) # for each group
@@ -323,6 +372,7 @@ lavaanify <- lavParTable <- function(
             MOD.label <- MOD[[el]]$label
             MOD.prior <- MOD[[el]]$prior
             MOD.efa   <- MOD[[el]]$efa
+            MOD.rv    <- MOD[[el]]$rv
 
             # check for single argument if multiple groups
             if(ngroups > 1L && length(idx) > 1L) {
@@ -334,17 +384,31 @@ lavaanify <- lavParTable <- function(
                 if(length(MOD.upper) == 1L) MOD.upper <- rep(MOD.upper, ngroups)
                 if(length(MOD.prior) == 1L) MOD.prior <- rep(MOD.prior, ngroups)
                 if(length(MOD.efa)   == 1L) MOD.efa   <- rep(MOD.efa,   ngroups)
+                if(length(MOD.rv)    == 1L) MOD.rv    <- rep(MOD.rv,    ngroups)
+
+                # new in 0.6-7 (proposal):
+                # - always recycle modifiers, including labels
+                # - if ngroups > 1 AND group.label= is empty, produce a warning
+                #   (as this is a break from < 0.6-6)
+                if(length(MOD.label) == 1L) {
+                    MOD.label <- rep(MOD.label, ngroups)
+                    if(is.null(group.equal) || length(group.equal) == 0L) {
+                        warn.about.single.label <- TRUE
+                    }
+                }
+
+                # < 0.6-7 code:
                 # B) here we do NOT! otherwise, it would imply an equality
                 #                    constraint...
                 #    except if group.equal="loadings"!
-                if(length(MOD.label) == 1L) {
-                    if("loadings" %in% group.equal ||
-                       "composite.loadings" %in% group.equal) {
-                        MOD.label <- rep(MOD.label, ngroups)
-                    } else {
-                        MOD.label <- c(MOD.label, rep("", (ngroups-1L)) )
-                    }
-                }
+                #if(length(MOD.label) == 1L) {
+                #    if("loadings" %in% group.equal ||
+                #       "composite.loadings" %in% group.equal) {
+                #        MOD.label <- rep(MOD.label, ngroups)
+                #    } else {
+                #        MOD.label <- c(MOD.label, rep("", (ngroups-1L)) )
+                #    }
+                #}
             }
 
             # check for wrong number of arguments if multiple groups
@@ -355,6 +419,7 @@ lavaanify <- lavParTable <- function(
                 (!is.null(MOD.upper) && nidx != length(MOD.upper)) ||
                 (!is.null(MOD.prior) && nidx != length(MOD.prior)) ||
                 (!is.null(MOD.efa)   && nidx != length(MOD.efa))   ||
+                (!is.null(MOD.rv)    && nidx != length(MOD.rv))    ||
                 (!is.null(MOD.label) && nidx != length(MOD.label)) ) {
                 el.idx <- which(LIST$mod.idx == el)[1L]
                 stop("lavaan ERROR: wrong number of arguments in modifier (",
@@ -393,6 +458,16 @@ lavaanify <- lavParTable <- function(
                 }
                 LIST$efa[idx] <- MOD.efa
             }
+            if(!is.null(MOD.rv)) {
+                # do we already have a `rv' column? if not, create one
+                if(is.null(LIST$rv)) {
+                    LIST$rv <- character( length(LIST$lhs) )
+                }
+                LIST$rv[idx] <- MOD.rv
+
+                LIST$free[  idx] <- 0L
+                LIST$ustart[idx] <- as.numeric(NA) #
+            }
             if(!is.null(MOD.lower)) {
                 # do we already have a `lower' column? if not, create one
                 if(is.null(LIST$lower)) {
@@ -415,6 +490,12 @@ lavaanify <- lavParTable <- function(
     # remove mod.idx column
     LIST$mod.idx <- NULL
 
+    # warning about single label in multiple group setting?
+    if(warn.about.single.label) {
+        warning("lavaan WARNING: using a single label per parameter in a multiple group\n",  "\t setting implies imposing equality constraints across all the groups;\n",    "\t If this is not intended, either remove the label(s), or use a vector\n",    "\t of labels (one for each group);\n",
+             "\t See the Multiple groups section in the man page of model.syntax.")
+    }
+
     # if lower/upper values were added, fix non-free values to ustart values
     # new in 0.6-6
     if(!is.null(LIST$lower)) {
@@ -430,6 +511,16 @@ lavaanify <- lavParTable <- function(
         }
     }
 
+    # if rv column is present, add RV.names to ALL rows where they are used
+    if(!is.null(LIST$rv)) {
+        RV.names <- unique(LIST$rv[ nchar(LIST$rv) > 0L ])
+        for(i in seq_len(length(RV.names))) {
+            lhs.idx <- which(LIST$lhs == RV.names[i] &
+                             LIST$op == "=~")
+            LIST$rv[lhs.idx] <- RV.names[i]
+        }
+    }
+
     if(debug) {
         cat("[lavaan DEBUG]: parameter LIST with MODIFIERS:\n")
         print( as.data.frame(LIST, stringsAsFactors=FALSE) )
@@ -477,25 +568,61 @@ lavaanify <- lavParTable <- function(
                 }
 
                 # equality constraints on ALL factor loadings in this set?
+                # two scenario's:
+                # 1. eq constraints within the same block, perhaps time1/time2/
+                # 2. eq constraints across groups (group.equal = "loadings")
                 # --> no constraints are needed
-                #if(b > 1L || s > 1L) {
-                #    # collect plabels for this set, if any
-                #    set.idx <- which(LIST$op == "=~" &
-                #                     LIST$block == b &
-                #                     LIST$lhs %in% lv.nam.efa)
-                #    plabels.set <- LIST$plabel[ set.idx ]
-                #
-                #    # check simple == equalities
-                #    # TODO: what about more complex equality constraints?
-                #    eq.idx <- which(LIST$op == "==")
-                #    con.labels <- c(LIST$lhs[eq.idx],
-                #                    LIST$rhs[eq.idx])
-                #
-                #    # TODO: what about exceptions?
-                #    if(all(plabels.set %in% con.labels)) {
-                #        next
-                #    }
-                #}
+
+                # store labels (if any)
+                fix.to.zero <- TRUE
+
+                # 1. within block/group
+                if(s == 1L) {
+                    set.idx <- which(LIST$op == "=~" &
+                                     LIST$block == b &
+                                     LIST$lhs %in% lv.nam.efa)
+                    LABEL.set1 <- LIST$label[set.idx]
+                } else {
+                    # collect plabels for this set, if any
+                    set.idx <- which(LIST$op == "=~" &
+                                     LIST$block == b &
+                                     LIST$lhs %in% lv.nam.efa)
+
+                    # user-provided labels (if any)
+                    this.label.set <- LIST$label[set.idx]
+
+                    # same as in reference set?
+                    if(all(nchar(this.label.set) > 0L) &&
+                       all(this.label.set %in% LABEL.set1)) {
+                        fix.to.zero <- FALSE
+                    }
+                }
+
+                # 2. across groups
+                if(b == 1L) {
+                    set.idx <- which(LIST$op == "=~" &
+                                     LIST$block == b &
+                                     LIST$lhs %in% lv.nam.efa)
+                    LABEL.group1 <- LIST$label[set.idx]
+                } else {
+                    if("loadings" %in% group.equal) {
+                       fix.to.zero <- FALSE
+                    } else {
+                        # collect labels for this set, if any
+                        set.idx <- which(LIST$op == "=~" &
+                                         LIST$block == b &
+                                         LIST$lhs %in% lv.nam.efa)
+
+                        # user-provided labels (if any)
+                        this.label.set <- LIST$label[set.idx]
+
+                        # same as in reference set?
+                        if(all(nchar(this.label.set) > 0L) &&
+                           all(this.label.set %in% LABEL.group1)) {
+                            fix.to.zero <- FALSE
+                        }
+                    }
+                }
 
                 # 1. echelon pattern
                 nfac <- length(lv.nam.efa)
@@ -513,10 +640,13 @@ lavaanify <- lavParTable <- function(
                     }
 
                     # fix to zero
-                    LIST$free[  ind.idx[seq_len(nzero)]] <- 0L
-                    LIST$ustart[ind.idx[seq_len(nzero)]] <- 0
-                    LIST$user[  ind.idx[seq_len(nzero)]] <- 7L
-
+                    if(fix.to.zero) {
+                        LIST$free[  ind.idx[seq_len(nzero)]] <- 0L
+                        LIST$ustart[ind.idx[seq_len(nzero)]] <- 0
+                        LIST$user[  ind.idx[seq_len(nzero)]] <- 7L
+                    } else {
+                        LIST$user[  ind.idx[seq_len(nzero)]] <- 77L
+                    }
                 }
 
                 # 2. covariances constrained to zero (only if oblique rotation)
@@ -530,9 +660,13 @@ lavaanify <- lavParTable <- function(
                                      LIST$lhs != LIST$rhs)
 
                     # fix to zero
-                    LIST$free[  cov.idx] <- 0L
-                    LIST$ustart[cov.idx] <- 0
-                    LIST$user[  cov.idx] <- 7L
+                    if(fix.to.zero) {
+                        LIST$free[  cov.idx] <- 0L
+                        LIST$ustart[cov.idx] <- 0
+                        LIST$user[  cov.idx] <- 7L
+                    } else {
+                        LIST$user[  cov.idx] <- 77L
+                    }
                 }
 
             } # sets
@@ -604,6 +738,18 @@ lavaanify <- lavParTable <- function(
                 # fix this one too
                 LIST$ustart[ref.idx] <- LIST$ustart[fixed.idx]
                 LIST$free[ref.idx] <- 0L
+
+                # new in 0.6-8 (for efa + user-specified eq constraints)
+                if(LIST$user[idx] %in% c(7L, 77L)) {
+                    # if involved in an efa block, store in CON anyway
+                    # we may need it for the rotated solution
+                    CON.idx <- CON.idx + 1L
+                    CON[[CON.idx]] <- list(op   = "==",
+                                           lhs  = LIST$plabel[ref.idx],
+                                           rhs  = LIST$plabel[idx],
+                                           user = 2L)
+                }
+
             } else {
             # 2. ref.idx is a free parameter
                 # user-label?
