@@ -7,6 +7,7 @@ lav_partable_unrestricted <- function(lavobject      = NULL,
                                       lavpta         = NULL,
                                       lavoptions     = NULL,
                                       lavsamplestats = NULL,
+                                      lavh1          = NULL,
                                       # optional user-provided sample stats
                                       sample.cov     = NULL,
                                       sample.mean    = NULL,
@@ -18,8 +19,9 @@ lav_partable_unrestricted <- function(lavobject      = NULL,
 
     lav_partable_indep_or_unrestricted(lavobject = lavobject,
         lavdata = lavdata, lavpta = lavpta, lavoptions = lavoptions,
-        lavsamplestats = lavsamplestats, sample.cov = sample.cov,
-        sample.mean = sample.mean , sample.slopes = sample.slopes,
+        lavsamplestats = lavsamplestats, lavh1 = lavh1,
+        sample.cov = sample.cov, sample.mean = sample.mean ,
+        sample.slopes = sample.slopes,
         sample.th = sample.th, sample.th.idx = sample.th.idx,
         independent = FALSE)
 
@@ -33,6 +35,7 @@ lav_partable_independence <- function(lavobject      = NULL,
                                       lavpta         = NULL,
                                       lavoptions     = NULL,
                                       lavsamplestats = NULL,
+                                      lavh1          = NULL,
                                       # optional user-provided sample stats
                                       sample.cov     = NULL,
                                       sample.mean    = NULL,
@@ -44,8 +47,9 @@ lav_partable_independence <- function(lavobject      = NULL,
 
     lav_partable_indep_or_unrestricted(lavobject = lavobject,
         lavdata = lavdata, lavpta = lavpta, lavoptions = lavoptions,
-        lavsamplestats = lavsamplestats, sample.cov = sample.cov,
-        sample.mean = sample.mean , sample.slopes = sample.slopes,
+        lavsamplestats = lavsamplestats, lavh1 = lavh1,
+        sample.cov = sample.cov, sample.mean = sample.mean ,
+        sample.slopes = sample.slopes,
         sample.th = sample.th, sample.th.idx = sample.th.idx,
         independent = TRUE)
 }
@@ -56,6 +60,7 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
                                       lavpta         = NULL,
                                       lavoptions     = NULL,
                                       lavsamplestats = NULL,
+                                      lavh1          = NULL,
                                       # optional user-provided sample stats
                                       sample.cov     = NULL,
                                       sample.mean    = NULL,
@@ -74,6 +79,7 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
         lavoptions <- lavobject@Options
         lavsamplestats <- lavobject@SampleStats
         lavpta <- lavobject@pta
+        lavh1 <- lavobject@h1
     }
 
     # conditional.x ? check res.cov[[1]] slot
@@ -84,6 +90,11 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
         conditional.x <- TRUE
     }
 
+    # group.w.free?
+    group.w.free <- FALSE
+    if(!is.null(lavoptions) && lavoptions$group.w.free) {
+        group.w.free <- TRUE
+    }
     # we use CAPS below for the list version, so we can use 'small caps'
     # within the for() loop
 
@@ -166,7 +177,6 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
 
         # only for multilevel
         if(nlevels > 1L) {
-            OV.NAMES <- lavdata@ov.names[[g]]
             YLp <-  lavsamplestats@YLp[[g]]
             Lp <- lavdata@Lp[[g]]
         }
@@ -209,26 +219,31 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
             if(nlevels > 1L) {
 
                 if(independent) {
-                    ov.names.x <- lavpta$vnames$ov.x[[b]]
+                    # beter use lavdata@Lp[[g]]$ov.x.idx??
+                    # in case we have x/y mismatch across levels?
+                    ov.x.idx <- lavpta$vidx$ov.x[[b]]
+                    ov.names.x <-   lavpta$vnames$ov.x[[b]]
                     ov.names.nox <- lavpta$vnames$ov.nox[[b]]
-                    sample.cov.x <- NULL # we will estimate them
-                    sample.mean.x <- NULL
+                    sample.cov.x <- lavh1$implied$cov[[b]][ov.x.idx,
+                                                      ov.x.idx, drop = FALSE]
+                    sample.mean.x <- lavh1$implied$mean[[b]][ov.x.idx]
                 } else {
                     ov.names.x <- character(0L)
                     ov.names.nox <- ov.names
                 }
 
-                block.idx <- match(ov.names, OV.NAMES)
+                sample.cov  <- lavh1$implied$cov[[b]]
+                sample.mean <- lavh1$implied$mean[[b]]
 
-                if(l == 1L) {
-                    sample.cov  <- YLp[[2]]$Sigma.W[block.idx, block.idx,
-                                                    drop = FALSE]
-                    sample.mean <- YLp[[2]]$Mu.W[block.idx]
-                } else {
-                    sample.cov  <- YLp[[2]]$Sigma.B[block.idx, block.idx,
-                                                    drop = FALSE]
-                    sample.mean <- YLp[[2]]$Mu.B[block.idx]
-                }
+                #if(l == 1L) {
+                #    sample.cov  <- YLp[[2]]$Sigma.W[block.idx, block.idx,
+                #                                    drop = FALSE]
+                #    sample.mean <- YLp[[2]]$Mu.W[block.idx]
+                #} else {
+                #    sample.cov  <- YLp[[2]]$Sigma.B[block.idx, block.idx,
+                #                                    drop = FALSE]
+                #    sample.mean <- YLp[[2]]$Mu.B[block.idx]
+                #}
 
                 # force local sample.cov to be strictly pd (and exaggerate)
                 # just for starting values anyway, but at least the first
@@ -555,7 +570,18 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
 
             } # conditional.x
 
-
+            # group.w.free (new in 0.6-8)
+            if(group.w.free) {
+                lhs    <- c(lhs, "group")
+                 op    <- c(op,  "%")
+                rhs    <- c(rhs, "w")
+                block  <- c(block, b)
+                group  <- c(group, g)
+                level  <- c(level, l)
+                free   <- c(free,  1L)
+                exo    <- c(exo,   0L)
+                ustart <- c(ustart, lavsamplestats@WLS.obs[[g]][1])
+            }
 
         } # levels
     } # ngroups
