@@ -12,6 +12,8 @@ lav_start <- function(start.method    = "default",
                       lavsamplestats  = NULL,
                       model.type      = "sem",
                       mimic           = "lavaan",
+                      reflect         = FALSE,    # rotation only
+                      order.lv.by     = "none",   # rotation only
                       debug           = FALSE) {
 
     # check arguments
@@ -28,6 +30,14 @@ lav_start <- function(start.method    = "default",
     # nlevels?
     nlevels <- lav_partable_nlevels(lavpartable)
 
+    # reflect/order.lv.by
+    if(is.null(reflect)) {
+        reflect <- FALSE
+    }
+    if(is.null(order.lv.by)) {
+        order.lv.by <- "index"
+    }
+
 
     # check start.method
     if(mimic == "lavaan") {
@@ -66,6 +76,12 @@ lav_start <- function(start.method    = "default",
         }
     } else if(is.list(start.method)) {
         start.user <- start.method
+    } else if(is.numeric(start.method)) {
+        nx.free <- sum(lavpartable$free > 0L)
+        if(length(start.method) != nx.free) {
+            stop("lavaan ERROR: start argument contains ", length(start.method), " elements; but parameter table expects ", nx.free, " free parameters.")
+        }
+        lavpartable$ustart[lavpartable$free > 0L] <- start.method
     } else if(inherits(start.method, "lavaan")) {
         start.user <- parTable(start.method)
     }
@@ -133,7 +149,7 @@ lav_start <- function(start.method    = "default",
         group.values <- lav_partable_group_values(lavpartable)
 
         # info from user model for this group
-        if(conditional.x) {
+        if(conditional.x && nlevels == 1L) {
             ov.names     <- vnames(lavpartable, "ov.nox", group = group.values[g])
         } else {
             ov.names     <- vnames(lavpartable, "ov", group = group.values[g])
@@ -163,10 +179,16 @@ lav_start <- function(start.method    = "default",
                             lavpartable$lhs == lavpartable$rhs)
         sample.var.idx <- match(lavpartable$lhs[ov.var.idx], ov.names)
         if(model.type == "unrestricted") {
-            start[ov.var.idx] <- diag(lavsamplestats@cov[[g]])[sample.var.idx]
+            if(!is.null(lavsamplestats@missing.h1[[g]])) {
+                start[ov.var.idx] <-
+                    diag(lavsamplestats@missing.h1[[g]]$sigma)[sample.var.idx]
+            } else {
+                start[ov.var.idx] <-
+                    diag(lavsamplestats@cov[[g]])[sample.var.idx]
+            }
         } else {
             if(start.initial == "mplus") {
-                if(conditional.x) {
+                if(conditional.x && nlevels == 1L) {
                     start[ov.var.idx] <-
                        (1.0 - 0.50)*lavsamplestats@res.var[[1L]][sample.var.idx]
                 } else {
@@ -174,7 +196,7 @@ lav_start <- function(start.method    = "default",
                        (1.0 - 0.50)*lavsamplestats@var[[1L]][sample.var.idx]
                 }
             } else {
-                if(conditional.x) {
+                if(conditional.x && nlevels == 1L) {
                     start[ov.var.idx] <-
                   (1.0 - 0.50)*diag(lavsamplestats@res.cov[[g]])[sample.var.idx]
                 } else {
@@ -218,7 +240,7 @@ lav_start <- function(start.method    = "default",
                         COV <- lavsamplestats@missing.h1[[g]]$sigma[ov.idx,
                                                       ov.idx, drop = FALSE]
                     } else {
-                        if(conditional.x) {
+                        if(conditional.x && nlevels == 1L) {
                             COV <- lavsamplestats@res.cov[[g]][ov.idx,
                                                       ov.idx, drop = FALSE]
                         } else {
@@ -235,6 +257,11 @@ lav_start <- function(start.method    = "default",
                     # factor loadings
                     tmp <- fabin$lambda
                     tmp[ !is.finite(tmp) ] <- 1.0 # just in case (eg 0/0)
+
+                    # check for negative triad if nvar=3L (new in 0.6-8)
+                    if(!is.null(fabin$neg.triad) && fabin$neg.triad) {
+                        tmp <- rep(1.0, length(tmp))
+                    }
                     start[lambda.idx] <- tmp
 
                     # factor variance
@@ -307,7 +334,7 @@ lav_start <- function(start.method    = "default",
                             COV <- lavsamplestats@missing.h1[[g]]$sigma[ov.idx,
                                                       ov.idx, drop = FALSE]
                         } else {
-                            if(conditional.x) {
+                            if(conditional.x && nlevels == 1L) {
                                 COV <- lavsamplestats@res.cov[[g]][ov.idx,
                                                           ov.idx, drop = FALSE]
                             } else {
@@ -317,8 +344,14 @@ lav_start <- function(start.method    = "default",
                          }
 
                         # EFA solution with zero upper-right corner
-                        EFA <- lav_efa_extraction_uls_corner(S = COV,
-                                              nfactors = length(lv.efa))
+                        EFA <- lav_efa_extraction(S = COV,
+                                              nfactors = length(lv.efa),
+                                              method = "ML",
+                                              order.lv.by = order.lv.by,
+                                              #order.lv.by = "none",
+                                              #reflect = reflect,
+                                              reflect = FALSE,
+                                              corner = TRUE)
 
                         # factor loadings
                         tmp <- as.numeric(EFA$LAMBDA)
@@ -336,12 +369,18 @@ lav_start <- function(start.method    = "default",
 
         if(model.type == "unrestricted") {
            # fill in 'covariances' from lavsamplestats
-            cov.idx <- which(lavpartable$group == group.values[g]             &
+            cov.idx <- which(lavpartable$group == group.values[g] &
                              lavpartable$op    == "~~"          &
                              lavpartable$lhs != lavpartable$rhs)
             lhs.idx <- match(lavpartable$lhs[cov.idx], ov.names)
             rhs.idx <- match(lavpartable$rhs[cov.idx], ov.names)
-            start[cov.idx] <- lavsamplestats@cov[[g]][ cbind(lhs.idx, rhs.idx) ]
+            if(!is.null(lavsamplestats@missing.h1[[g]])) {
+                start[cov.idx] <- lavsamplestats@missing.h1[[g]]$sigma[
+                                                 cbind(lhs.idx, rhs.idx) ]
+            } else {
+                start[cov.idx] <- lavsamplestats@cov[[g]][
+                                                 cbind(lhs.idx, rhs.idx) ]
+            }
         }
 
         # variances of ordinal variables - set to 1.0
@@ -361,7 +400,7 @@ lav_start <- function(start.method    = "default",
         if(lavsamplestats@missing.flag && nlevels == 1L) {
             start[ov.int.idx] <- lavsamplestats@missing.h1[[g]]$mu[sample.int.idx]
         } else {
-            if(conditional.x) {
+            if(conditional.x && nlevels == 1L) {
                 start[ov.int.idx] <- lavsamplestats@res.int[[g]][sample.int.idx]
             } else {
                 start[ov.int.idx] <- lavsamplestats@mean[[g]][sample.int.idx]
@@ -377,7 +416,7 @@ lav_start <- function(start.method    = "default",
                 lavsamplestats@th.names[[g]][ lavsamplestats@th.idx[[g]] > 0L ]
             # th.names.sample should identical to
             # vnames(lavpartable, "th", group = group.values[g])
-            if(conditional.x) {
+            if(conditional.x && nlevels == 1L) {
                 th.values <-
                  lavsamplestats@res.th[[g]][lavsamplestats@th.idx[[g]] > 0L]
             } else {
@@ -451,7 +490,7 @@ lav_start <- function(start.method    = "default",
                          single.var <- 1
                     }
                     ov.idx <- match(single.ind, ov.names)
-                    if(conditional.x) {
+                    if(conditional.x && nlevels == 1L) {
                         ov.var <- diag(lavsamplestats@res.cov[[g]])[ov.idx]
                     } else {
                         ov.var <- diag(lavsamplestats@cov[[g]])[ov.idx]
@@ -658,7 +697,9 @@ lav_start <- function(start.method    = "default",
     }
 
     # override if the model syntax contains explicit starting values
-    user.idx <- which(!is.na(lavpartable$ustart))
+    user.idx <- which(!is.na(lavpartable$ustart) &
+                      lavpartable$user != 7L) # new in 0.6-7, if rotation and
+                                              # and we change the order of lv's
     start[user.idx] <- lavpartable$ustart[user.idx]
 
     # final check: no NaN or other non-finite values
