@@ -312,7 +312,7 @@ lav_test_diff_m10 <- function(m1, m0, test = FALSE) {
     Options$optim.method          = "none"
     Options$optim.force.converged = TRUE
     Options$baseline              = FALSE
-    Options$h1                    = FALSE
+    Options$h1                    = TRUE # needed after all (yuan.benter.mplus)
     Options$start                 = PT.M0.extended # new in 0.6!
     m10 <- lavaan(model = PT.M1.extended,
                   slotOptions     = Options,
