@@ -28,6 +28,27 @@ lav_test_yuan_bentler <- function(lavobject      = NULL,
         TEST$standard  <- TEST.unscaled
     }
 
+    # E.inv ok?
+    if( length(lavoptions$information) == 1L &&
+        length(lavoptions$h1.information) == 1L &&
+        length(lavoptions$observed.information) == 1L) {
+        E.inv.recompute <- FALSE
+    } else if( (lavoptions$information[1] == lavoptions$information[2]) &&
+        (lavoptions$h1.information[1] == lavoptions$h1.information[2]) &&
+        (lavoptions$observed.information[1] ==
+         lavoptions$observed.information[2]) ) {
+        E.inv.recompute <- FALSE
+    } else {
+        E.inv.recompute <- TRUE
+        # change information options
+        lavoptions$information[1] <- lavoptions$information[2]
+        lavoptions$h1.information[1] <- lavoptions$h1.information[2]
+        lavoptions$observed.information[1] <- lavoptions$observed.information[2]
+    }
+    if(!is.null(E.inv)) {
+        E.inv.recompute <- FALSE # user-provided
+    }
+
     # check test
     if(!all(test %in% c("yuan.bentler",
                         "yuan.bentler.mplus"))) {
@@ -36,21 +57,14 @@ lav_test_yuan_bentler <- function(lavobject      = NULL,
     }
 
     # information
-    information <- lavoptions$information
-
-    # x.idx
-    if(lavoptions$conditional.x) {
-        x.idx <- NULL
-    } else {
-        x.idx <- lavsamplestats@x.idx
-    }
+    information <- lavoptions$information[1]
 
     # ndat
     ndat <- numeric(lavsamplestats@ngroups)
 
 
     # do we have E.inv?
-    if(is.null(E.inv)) {
+    if(is.null(E.inv) || E.inv.recompute) {
         E.inv <- try(lav_model_information(lavmodel       = lavmodel,
                                            lavsamplestats = lavsamplestats,
                                            lavdata        = lavdata,
@@ -72,6 +86,7 @@ lav_test_yuan_bentler <- function(lavobject      = NULL,
     if(TEST$standard$df == 0L || TEST$standard$df < 0) {
         TEST[[test[1]]] <- c(TEST$standard, scaling.factor = as.numeric(NA),
                              label = character(0))
+        TEST[[test[1]]]$test <- test[1] # to prevent lavTestLRT error when robust test is detected for some but not all models
         return(TEST)
     }
 
@@ -125,19 +140,29 @@ lav_test_yuan_bentler <- function(lavobject      = NULL,
                                               B1.group       = B1.group,
                                               B0.group       = B0.group,
                                               E.inv          = E.inv,
-                                              x.idx          = x.idx,
                                               meanstructure  = lavmodel@meanstructure)
     } else if(test == "yuan.bentler") {
 
+        # compute Delta
         Delta <- computeDelta(lavmodel = lavmodel)
+
+        # compute Omega/Gamma
+        Omega <- lav_model_h1_omega(lavmodel       = lavmodel,
+                                    lavsamplestats = lavsamplestats,
+                                    lavdata        = lavdata,
+                                    lavimplied     = lavimplied,
+                                    lavh1          = lavh1,
+                                    lavoptions     = lavoptions)
+
+        # compute trace 'U %*% Gamma' (or 'U %*% Omega')
         trace.UGamma <- lav_test_yuan_bentler_trace(
             lavsamplestats = lavsamplestats,
             meanstructure  = lavmodel@meanstructure,
             A1.group       = A1.group,
             B1.group       = B1.group,
             Delta          = Delta,
+            Omega          = Omega,
             E.inv          = E.inv,
-            x.idx          = x.idx,
             Satterthwaite  = TRUE) # for now
     }
 
@@ -197,8 +222,8 @@ lav_test_yuan_bentler_trace <- function(lavsamplestats =lavsamplestats,
                                         A1.group       = NULL,
                                         B1.group       = NULL,
                                         Delta          = NULL,
+                                        Omega          = NULL,
                                         E.inv          = NULL,
-                                        x.idx          = list(integer(0)),
                                         Satterthwaite  = FALSE) {
 
     # we always assume a meanstructure (nope, not any longer, since 0.6)
@@ -206,41 +231,32 @@ lav_test_yuan_bentler_trace <- function(lavsamplestats =lavsamplestats,
 
     trace.UGamma  <- numeric( lavsamplestats@ngroups )
     trace.UGamma2 <- numeric( lavsamplestats@ngroups )
-    trace.h1      <- numeric( lavsamplestats@ngroups )
     trace.h0      <- numeric( lavsamplestats@ngroups )
-    h1.ndat       <- numeric( lavsamplestats@ngroups )
+
+    trace.h1      <- attr(Omega, "trace.h1")
+    h1.ndat       <- attr(Omega, "h1.ndat")
 
     for(g in 1:lavsamplestats@ngroups) {
 
         # group weight
         fg <- lavsamplestats@nobs[[g]]/lavsamplestats@ntotal
 
-        A1 <- A1.group[[g]]
-        B1 <- B1.group[[g]]
+        A1 <- A1.group[[g]] * fg
+        B1 <- B1.group[[g]] * fg
         DELTA <- Delta[[g]]
-
-        # mask independent 'fixed-x' variables
-        zero.idx <- which(diag(A1) == 0)
-        if(length(zero.idx) > 0L) {
-            A1.inv <- matrix(0, nrow(A1), ncol(A1))
-            a1 <- A1[-zero.idx, -zero.idx]
-            a1.inv <- solve(a1)
-            A1.inv[-zero.idx, -zero.idx] <- a1.inv
-        } else {
-            A1.inv <- solve(A1)
-        }
-        h1.ndat[g] <- ncol(A1) - length(zero.idx)
+        Gamma.g <- Omega[[g]] / fg
 
         D.Einv.tD <- DELTA %*% tcrossprod(E.inv, DELTA)
 
-        trace.h1[g] <- sum( B1 * t( A1.inv ) )
+        #trace.h1[g] <- sum( B1 * t( A1.inv ) )
         # fg cancels out: trace.h1[g] <- sum( fg*B1 * t( 1/fg*A1.inv ) )
-        trace.h0[g] <- fg * sum( B1 * D.Einv.tD )
-        trace.UGamma[g] <- trace.h1[g] - trace.h0[g]
+        trace.h0[g] <- sum( B1 * D.Einv.tD )
+        #trace.UGamma[g] <- trace.h1[g] - trace.h0[g]
+        U <- A1 - A1 %*% D.Einv.tD %*% A1
+        trace.UGamma[g] <- sum(U * Gamma.g)
 
         if(Satterthwaite) {
-            A1invB1 <- A1.inv %*% B1
-            UG <- A1invB1 - fg * (A1invB1 %*% D.Einv.tD %*% A1)
+            UG <- U %*% Gamma.g
             trace.UGamma2[g] <- sum(UG * t(UG))
         }
     }
@@ -263,7 +279,6 @@ lav_test_yuan_bentler_mplus_trace <- function(lavsamplestats=NULL,
                                               B1.group = NULL,
                                               B0.group=NULL,
                                               E.inv=NULL,
-                                              x.idx=list(integer(0)),
                                               meanstructure = TRUE) {
     # typical for Mplus:
     # - do NOT use the YB formula, but use an approximation
