@@ -237,6 +237,20 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         lv.names   <- unique(unlist(lav_partable_vnames(FLAT, type = "lv")))
     }
 
+    # sanity check (new in 0.6-8): do we have any ov.names?
+    # detect early
+    if(length(ov.names) == 0L) {
+        stop("lavaan ERROR: ov.names is empty: model does not refer to any observed variables; check your syntax.")
+    }
+
+    # sanity check: ov.names.x should NOT appear in ov.names.y
+    # this may happen if 'x' is exogenous in one block, but not in another...
+    #endo.idx <- which(ov.names.x %in% ov.names.y)
+    #if(length(endo.idx) > 0L) {
+    #    # remove from x! (new in 0.6-8)
+    #    ov.names.x <- ov.names.x[-endo.idx]
+    #}
+
     # handle for lv.names that are also observed variables (new in 0.6-6)
     LV.names <- unique(unlist(lv.names))
     if(length(LV.names) > 0L) {
@@ -603,17 +617,21 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             print(as.data.frame(FLAT))
         }
         # catch ~~ of fixed.x covariates if fixed.x = TRUE
-        if(lavoptions$fixed.x) {
-            tmp <- try(vnames(FLAT, type = "ov.x", ov.x.fatal = TRUE),
-                       silent = TRUE)
-            if(inherits(tmp, "try-error")) {
-                warning("lavaan WARNING: syntax contains parameters involving exogenous covariates; switching to fixed.x = FALSE")
-                lavoptions$fixed.x <- FALSE
-            }
-        }
-        if(lavoptions$conditional.x) {
-            tmp <- vnames(FLAT, type = "ov.x", ov.x.fatal = TRUE)
-        }
+        # --> done inside lavaanify!
+
+        #if(lavoptions$fixed.x) {
+        #    tmp <- lav_partable_vnames(FLAT, type = "ov.x",
+        #                               ov.x.fatal = FALSE, warn = TRUE)
+            #tmp <- try(vnames(FLAT, type = "ov.x", ov.x.fatal = TRUE),
+            #           silent = TRUE)
+            #if(inherits(tmp, "try-error")) {
+            #    warning("lavaan WARNING: syntax contains parameters involving exogenous covariates; switching to fixed.x = FALSE")
+            #    lavoptions$fixed.x <- FALSE
+            #}
+        #}
+        #if(lavoptions$conditional.x) {
+        #    tmp <- vnames(FLAT, type = "ov.x", ov.x.fatal = TRUE)
+        #}
 
         lavpartable <-
             lavaanify(model            = FLAT,
@@ -702,28 +720,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     } else if(lavdata@data.type == "full") {
         lavsamplestats <- lav_samplestats_from_data(
                        lavdata       = lavdata,
-                       missing       = lavoptions$missing,
-                       rescale       =
-                           (lavoptions$estimator %in% c("ML","REML","NTRLS") &&
-                            lavoptions$likelihood == "normal"),
-                       estimator     = lavoptions$estimator,
-                       mimic         = lavoptions$mimic,
-                       meanstructure = lavoptions$meanstructure,
-                       conditional.x = lavoptions$conditional.x,
-                       fixed.x       = lavoptions$fixed.x,
-                       group.w.free  = lavoptions$group.w.free,
-                       missing.h1    = (lavoptions$missing != "listwise"),
-                       WLS.V             = WLS.V,
-                       NACOV             = NACOV,
-                       gamma.n.minus.one = lavoptions$gamma.n.minus.one,
-                       se                = lavoptions$se,
-                       test              = lavoptions$test,
-                       ridge             = lavoptions$ridge,
-                       zero.add          = lavoptions$zero.add,
-                       zero.keep.margins = lavoptions$zero.keep.margins,
-                       zero.cell.warn    = lavoptions$zero.cell.warn,
-                       debug             = lavoptions$debug,
-                       verbose           = lavoptions$verbose)
+                       lavoptions    = lavoptions,
+                       WLS.V         = WLS.V,
+                       NACOV         = NACOV)
 
     } else if(lavdata@data.type == "moment") {
         lavsamplestats <- lav_samplestats_from_moments(
@@ -864,13 +863,11 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
 
             lavpartable$start <- START
         }
-
-
         timing$start <- (proc.time()[3] - start.time)
         start.time <- proc.time()[3]
 
 
-        # 8b. bounds for EFA -- change user==7 element if columns
+        # 8b. EFA -- change user==7 elements if columns
         #     have been reordered
         if( !is.null(lavpartable$efa) && any(lavpartable$user == 7L) &&
             lavoptions$rotation != "none" ) {
@@ -880,7 +877,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                          lavpartable$op == "=~" &
                          abs(lavpartable$start) > sqrt(.Machine$double.eps))
             if(length(idx) > 0L) {
-                lavpartable$user[idx] <- 0L
+                lavpartable$user[idx] <- 1L
                 lavpartable$free[idx] <- 1L
                 lavpartable$ustart[idx] <- as.numeric(NA)
             }
@@ -902,7 +899,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             if(length(idx) > 0L) {
                 lavpartable$free[idx] <- seq_len(length(idx))
             }
-        }
+        } # EFA
 
 
 
@@ -957,33 +954,32 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
 
 
     # 9b. bounds for EFA -- to force diag(LAMBDA) to be positive (new in 0.6-7)
-    if( (.hasSlot(lavmodel, "nefa")) && (lavmodel@nefa > 0L) &&
-        (lavoptions$rotation != "none") ) {
-
-        # add lower column
-        if(is.null(lavpartable$lower)) {
-            lavpartable$lower <- rep(-Inf, length(lavpartable$lhs))
-        }
-        efa.values <- lav_partable_efa_values(lavpartable)
-        group.values <- lav_partable_group_values(lavpartable)
-        for(g in seq_len(lavdata@ngroups)) {
-            for(set in seq_len(lavmodel@nefa)) {
-                lv.efa <-
-                    unique(lavpartable$lhs[ lavpartable$op == "=~" &
-                                            lavpartable$block == g &
-                                            lavpartable$efa == efa.values[set]])
-                for(f in seq_len(length(lv.efa))) {
-                    lambda.idx <- which( lavpartable$lhs == lv.efa[f] &
-                                         lavpartable$op == "=~" &
-                                         lavpartable$group == group.values[g] )
-                    # get largest factor loading
-                    #midx <- lambda.idx[which.max(lavpartable$start[lambda.idx])]
-                    midx <- lambda.idx[f] # diagonal element of LAMBDA
-                    lavpartable$lower[midx] <- 0
-                 } # factors
-            } # sets
-        } # groups
-    }
+    #if( (.hasSlot(lavmodel, "nefa")) && (lavmodel@nefa > 0L) &&
+    #    (lavoptions$rotation != "none") ) {
+    #
+    #    # add lower column
+    #    if(is.null(lavpartable$lower)) {
+    #        lavpartable$lower <- rep(-Inf, length(lavpartable$lhs))
+    #    }
+    #    efa.values <- lav_partable_efa_values(lavpartable)
+    #    group.values <- lav_partable_group_values(lavpartable)
+    #    for(g in seq_len(lavdata@ngroups)) {
+    #        for(set in seq_len(lavmodel@nefa)) {
+    #            lv.efa <-
+    #                unique(lavpartable$lhs[lavpartable$op == "=~" &
+    #                                       lavpartable$block == g &
+    #                                       lavpartable$efa == efa.values[set] ])
+    #            for(f in seq_len(length(lv.efa))) {
+    #                lambda.idx <- which( lavpartable$lhs == lv.efa[f] &
+    #                                     lavpartable$op == "=~" &
+    #                                     lavpartable$group == group.values[g] )
+    #                # get diagonal element of LAMBDA
+    #                midx <- lambda.idx[f] # diagonal element of LAMBDA
+    #                lavpartable$lower[midx] <- 0
+    #             } # factors
+    #        } # sets
+    #    } # groups
+    #}
 
 
 
@@ -1185,10 +1181,11 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     if(lavoptions$do.fit && lavoptions$estimator != "none" &&
        lavmodel@nx.free > 0L) {
 
+        # EM for multilevel models
         if(lavoptions$optim.method == "em") {
             # multilevel only for now
             stopifnot(lavdata@nlevels > 1L)
-            x <- lav_mvnorm_cluster_em_h0(lavsamplestats = lavsamplestats,
+            x <- try(lav_mvnorm_cluster_em_h0(lavsamplestats = lavsamplestats,
                                lavdata        = lavdata,
                                lavimplied     = NULL,
                                lavpartable    = lavpartable,
@@ -1197,53 +1194,98 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                                verbose        = lavoptions$verbose,
                                fx.tol         = lavoptions$em.fx.tol,
                                dx.tol         = lavoptions$em.dx.tol,
-                               max.iter       = lavoptions$em.iter.max)
+                               max.iter       = lavoptions$em.iter.max),
+                      silent = TRUE)
+        # Gauss-Newton
+        } else if(lavoptions$optim.method == "gn") {
+            # only tested for DLS (for now)
+            #if(lavoptions$estimator != "DLS") {
+            #    stop("lavaan ERROR: optim.method = ", dQuote("gn"),
+            #         " is only available for estimator = ", dQuote("DLS"),
+            #         " (for now).")
+            #}
+            x <- try(lav_optim_gn(lavmodel       = lavmodel,
+                                  lavsamplestats = lavsamplestats,
+                                  lavdata        = lavdata,
+                                  lavpartable    = lavpartable,
+                                  lavoptions     = lavoptions),
+                     silent = TRUE)
+
+        # Quasi-Newton
         } else {
             # try 1
-            x <- lav_model_estimate(lavmodel        = lavmodel,
-                                    lavpartable     = lavpartable,
-                                    lavsamplestats  = lavsamplestats,
-                                    lavdata         = lavdata,
-                                    lavoptions      = lavoptions,
-                                    lavcache        = lavcache)
-
-            # try 2: optim.parscale = "standardize" (new in 0.6-7)
-            if(!attr(x, "converged")) {
-                lavoptions2 <- lavoptions
-                lavoptions2$optim.parscale = "standardized"
-                x <- lav_model_estimate(lavmodel        = lavmodel,
-                                        lavpartable     = lavpartable,
-                                        lavsamplestats  = lavsamplestats,
-                                        lavdata         = lavdata,
-                                        lavoptions      = lavoptions2,
-                                        lavcache        = lavcache)
-            }
-
-            # try 3: start = "simple"
-            if(!attr(x, "converged")) {
-                x <- lav_model_estimate(lavmodel        = lavmodel,
+            x <- try(lav_model_estimate(lavmodel        = lavmodel,
                                         lavpartable     = lavpartable,
                                         lavsamplestats  = lavsamplestats,
                                         lavdata         = lavdata,
                                         lavoptions      = lavoptions,
-                                        start           = "simple",
-                                        lavcache        = lavcache)
+                                        lavcache        = lavcache),
+                     silent = TRUE)
+            # store first attempt
+            x.first <- x
+
+            # try 2: optim.parscale = "standardize" (new in 0.6-7)
+            if(lavoptions$optim.attempts > 1L &&
+               (inherits(x, "try-error") || !attr(x, "converged"))) {
+                lavoptions2 <- lavoptions
+                lavoptions2$optim.parscale = "standardized"
+                x <- try(lav_model_estimate(lavmodel        = lavmodel,
+                                            lavpartable     = lavpartable,
+                                            lavsamplestats  = lavsamplestats,
+                                            lavdata         = lavdata,
+                                            lavoptions      = lavoptions2,
+                                            lavcache        = lavcache),
+                         silent = TRUE)
+            }
+
+            # try 3: start = "simple"
+            if(lavoptions$optim.attempts > 2L &&
+               (inherits(x, "try-error") || !attr(x, "converged"))) {
+                x <- try(lav_model_estimate(lavmodel        = lavmodel,
+                                            lavpartable     = lavpartable,
+                                            lavsamplestats  = lavsamplestats,
+                                            lavdata         = lavdata,
+                                            lavoptions      = lavoptions,
+                                            start           = "simple",
+                                            lavcache        = lavcache),
+                         silent = TRUE)
             }
 
             # try 4: start = "simple" + optim.parscale = "standardize"
-            if(!attr(x, "converged")) {
+            if(lavoptions$optim.attempts > 3L &&
+               (inherits(x, "try-error") || !attr(x, "converged"))) {
                 lavoptions2 <- lavoptions
                 lavoptions2$optim.parscale = "standardized"
-                x <- lav_model_estimate(lavmodel        = lavmodel,
-                                        lavpartable     = lavpartable,
-                                        lavsamplestats  = lavsamplestats,
-                                        lavdata         = lavdata,
-                                        lavoptions      = lavoptions2,
-                                        start           = "simple",
-                                        lavcache        = lavcache)
+                x <- try(lav_model_estimate(lavmodel        = lavmodel,
+                                            lavpartable     = lavpartable,
+                                            lavsamplestats  = lavsamplestats,
+                                            lavdata         = lavdata,
+                                            lavoptions      = lavoptions2,
+                                            start           = "simple",
+                                           lavcache        = lavcache),
+                         silent = TRUE)
             }
+        }
 
+        # optimization failed with error
+        if(inherits(x, "try-error")) {
+            warn.txt <- "Model estimation FAILED! Returning starting values."
+            x <- lav_model_get_parameters(lavmodel = lavmodel,
+                                          type = "free") # starting values
+            attr(x, "iterations") <- 0L
+            attr(x, "converged") <- FALSE
+            attr(x, "warn.txt") <- warn.txt
+            attr(x, "control") <- lavoptions$control
+            attr(x, "dx") <- numeric(0L)
+            fx <- as.numeric(NA)
+            attr(fx, "fx.group") <-  as.numeric(NA)
+            attr(x, "fx") <- fx
+        }
 
+        # if a warning was produced, say it here
+        warn.txt <- attr(x, "warn.txt")
+        if(lavoptions$warn && nchar(warn.txt) > 0L) {
+            warning(lav_txt2message(warn.txt))
         }
 
         # in case of non-linear constraints: store final con.jac and con.lambda
@@ -1263,6 +1305,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     } else {
         x <- numeric(0L)
         attr(x, "iterations") <- 0L; attr(x, "converged") <- FALSE
+        attr(x, "warn.txt") <- ""
         attr(x, "control") <- lavoptions$control
         attr(x, "dx") <- numeric(0L)
         attr(x, "fx") <-
@@ -1287,6 +1330,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     lavoptim$npar <- length(x)
     lavoptim$iterations <- attr(x, "iterations")
     lavoptim$converged  <- attr(x, "converged")
+    lavoptim$warn.txt   <- attr(x, "warn.txt")
     lavoptim$parscale   <- attr(x, "parscale")
     fx.copy <- fx <- attr(x, "fx"); attributes(fx) <- NULL
     lavoptim$fx         <- fx
@@ -1332,7 +1376,12 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     VCOV <- NULL
     if(lavoptions$se != "none" && lavoptions$se != "external" &&
        lavoptions$se != "twostep" &&
-       lavmodel@nx.free > 0L && attr(x, "converged")) {
+       ( .hasSlot(lavmodel, "nefa") &&
+           (  lavmodel@nefa == 0L ||
+              (lavmodel@nefa > 0L && lavoptions$rotation == "none") ||
+              (lavmodel@nefa > 0L && lavoptions$rotation.se == "delta")
+           )
+       ) && lavmodel@nx.free > 0L && attr(x, "converged")) {
 
         if(lavoptions$verbose) {
             cat("Computing VCOV for se =", lavoptions$se, "...")
@@ -1380,18 +1429,20 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                     vcov = VCOV1)
 
     # store se in partable
-    if(lavoptions$se != "external" && lavoptions$se != "twostep") {
-        lavpartable$se <- lav_model_vcov_se(lavmodel = lavmodel,
-                                            lavpartable = lavpartable,
-                                            VCOV = VCOV,
-                                            BOOT = lavboot$coef)
-    } else {
+    if(lavoptions$se == "external") {
         if(is.null(lavpartable$se)) {
             lavpartable$se <- lav_model_vcov_se(lavmodel = lavmodel,
                                                 lavpartable = lavpartable,
                                                 VCOV = NULL, BOOT = NULL)
             warning("lavaan WARNING: se = \"external\" but parameter table does not contain a `se' column")
         }
+    } else if(lavoptions$se == "twostep") {
+       # do nothing
+    } else {
+        lavpartable$se <- lav_model_vcov_se(lavmodel = lavmodel,
+                                            lavpartable = lavpartable,
+                                            VCOV = VCOV,
+                                            BOOT = lavboot$coef)
     }
 
     timing$vcov <- (proc.time()[3] - start.time)
@@ -1518,94 +1569,161 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             cat("Rotating EFA factors using rotation method =",
                 toupper(lavoptions$rotation), "... ")
         }
-
-        # pre-rotate: fix 'sign' and 'order', according to the options
-        # in lavoptions; this is important for the 'order' of the parameters
-        # in vcov, once we use the delta rule
-        #lavmodel <- lav_model_efa_rotate_pre(lavmodel = lavmodel,
-        #                                     lavoptions = lavoptions)
-        #x.unrotated <- as.numeric(lav_model_get_parameters(lavmodel))
         x.unrotated <- as.numeric(x)
-
-
         lavmodel.unrot <- lavmodel
-        lavmodel <- lav_model_efa_rotate(lavmodel = lavmodel,
+        lavmodel <- lav_model_efa_rotate(lavmodel   = lavmodel,
                                          lavoptions = lavoptions)
+        # overwrite parameters in @ParTable$est
+        lavpartable$est <- lav_model_get_parameters(lavmodel = lavmodel,
+                                                    type = "user", extra = TRUE)
         if(lavoptions$verbose) {
             cat("done.\n")
         }
 
-        # overwrite parameters in @ParTable$est
-        lavpartable$est <- lav_model_get_parameters(lavmodel = lavmodel,
-                                                    type = "user", extra = TRUE)
-
-        if(!lavoptions$se %in% c("none", "bootstrap")) {
-            # use delta rule to recompute vcov
-
+        # VCOV rotated parameters
+        if(!lavoptions$se %in% c("none", "bootstrap", "external", "two.step")) {
             if(lavoptions$verbose) {
-                cat("Using delta method to compute VCOV of rotated parameters ...")
+                cat("Computing VCOV for se =", lavoptions$se,
+                    "and rotation.se =", lavoptions$rotation.se, "...")
             }
 
-            # Jacobian
-            JAC <- numDeriv::jacobian(func = lav_model_efa_rotate_x,
-                                      x = x.unrotated,
-                                      # Note: lavmodel MUST be UNROTATED
-                                      lavmodel = lavmodel.unrot,
-                                      init.rot = lavmodel@H,
-                                      #jacobian = TRUE,
-                                      lavoptions = lavoptions,
-                                      type = "user",
-                                      extra = FALSE,
-                                      method.args = list(eps = 1e-03),
-                                      method = "simple") # to save time
+            # use delta rule to recompute vcov
+            if(lavoptions$rotation.se == "delta") {
+                # Jacobian
+                JAC <- numDeriv::jacobian(func = lav_model_efa_rotate_x,
+                  x = x.unrotated, lavmodel = lavmodel.unrot,
+                  init.rot = lavmodel@H, lavoptions = lavoptions,
+                  type = "user", extra = FALSE,
+                  method.args = list(eps = 0.0050),
+                  method = "simple") # important!
 
-            # compute unrotated VCOV (possibly after re-ordering the factors!)
-            #VCOV <- lav_model_vcov(lavmodel        = lavmodel.unrot,
-            #                       lavsamplestats  = lavsamplestats,
-            #                       lavoptions      = lavoptions,
-            #                       lavdata         = lavdata,
-            #                       lavpartable     = lavpartable,
-            #                       lavcache        = lavcache,
-            #                       lavimplied      = lavimplied,
-            #                       lavh1           = lavh1)
+                # force VCOV to be pd, before we transform (not very elegant)
+                VCOV.in <- lav_matrix_symmetric_force_pd(lavvcov$vcov,
+                                                         tol = 1e-10)
+                #VCOV.in <- as.matrix(Matrix:::nearPD(x = lavvcov$vcov)$mat)
 
-            # force VCOV to be pd, before we transform
-            #VCOV.in <- lav_matrix_symmetric_force_pd(VCOV)
-            #lavvcov <- list(se = lavoptions$se,
-            #                information = lavoptions$information,
-            #                vcov = VCOV.in)
+                # apply Delta rule
+                VCOV.user <- JAC %*% VCOV.in %*% t(JAC)
 
-            VCOV.in <- lav_matrix_symmetric_force_pd(lavvcov$vcov, tol = 1e-06)
-            #VCOV.in <- lavvcov$vcov
-
-            # apply Delta rule
-            VCOV.user <- JAC %*% VCOV.in %*% t(JAC)
-
-            # re-compute SE and store them in lavpartable
-            if(lavoptions$se != "external" && lavoptions$se != "twostep") {
+                # re-compute SE and store them in lavpartable
                 tmp <- diag(VCOV.user)
-                # catch negative values
                 min.idx <- which(tmp < 0)
                 if(length(min.idx) > 0L) {
                     tmp[min.idx] <- as.numeric(NA)
                 }
-                # now, we can safely take the square root
                 tmp <- sqrt(tmp)
-
-                # catch near-zero SEs
-                zero.idx <- which(tmp < .Machine$double.eps^(1/3)) # was 1/2 < 0.6
+                # catch near-zero SEs  ( was ^(1/2) < 0.6 )
+                zero.idx <- which(tmp < .Machine$double.eps^(1/3))
                 if(length(zero.idx) > 0L) {
                     tmp[zero.idx] <- 0.0
                 }
-
                 lavpartable$se <- tmp
             }
 
-            if(lavoptions$verbose) {
-                cat("done.\n")
+            else if(lavoptions$rotation.se == "bordered") {
+                # create 'new' partable where the user=7/77 parameters are free
+                PT.new <- lavpartable
+
+                user7.idx <- which(PT.new$user == 7L |
+                                   PT.new$user == 77L)
+                PT.new$free[user7.idx] <- 1L
+                PT.new$free[ PT.new$free > 0L ] <-
+                                         seq_len(sum(PT.new$free > 0L))
+
+                # create 'new' lavmodel (where user7/77 parameters are free)
+                lavmodel.new <- lav_model(lavpartable = PT.new,
+                                          lavoptions = lavoptions,
+                                          th.idx     = lavmodel@th.idx)
+                lavmodel.new@GLIST    <- lavmodel@GLIST
+                lavmodel.new@H        <- lavmodel@H
+                lavmodel.new@lv.order <- lavmodel@lv.order
+
+                # create 'border' for augmented information matrix
+                x.rot <- lav_model_get_parameters(lavmodel.new)
+                JAC <- numDeriv::jacobian(func = lav_model_efa_rotate_border_x,
+                                 x = x.rot, lavmodel = lavmodel.new,
+                                 lavoptions = lavoptions,
+                                 lavpartable = lavpartable,
+                                 #method.args = list(eps = 0.0005),
+                                 #method = "simple")
+                                 method = "Richardson")
+                # store JAC
+                lavmodel.new@ceq.efa.JAC <- JAC
+
+                # no other constraints
+                if(length(lavmodel@ceq.linear.idx) == 0L &&
+                   length(lavmodel@ceq.nonlinear.idx) == 0L &&
+                   length(lavmodel@cin.linear.idx)    == 0L &&
+                   length(lavmodel@cin.nonlinear.idx) == 0L) {
+                    lavmodel.new@con.jac <- JAC
+                    attr(lavmodel.new@con.jac, "inactive.idx") <- integer(0L)
+                    attr(lavmodel.new@con.jac, "ceq.idx") <- seq_len(nrow(JAC))
+                    attr(lavmodel.new@con.jac, "cin.idx") <- integer(0L)
+                    lavmodel.new@con.lambda <- rep(0, nrow(JAC))
+
+                # other constraints
+                } else {
+                    inactive.idx <- attr(lavmodel@con.jac, "inactive.idx")
+                    ceq.idx <- attr(lavmodel@con.jac, "ceq.idx")
+                    cin.idx <- attr(lavmodel@con.jac, "cin.idx")
+                    lambda <- lavmodel@con.lambda
+                    nbord <- nrow(JAC)
+
+                    # recompute con.jac
+                    if(is.null(body(lavmodel.new@ceq.function))) {
+                        ceq <- function(x, ...) { return( numeric(0) ) }
+                    } else {
+                        ceq <- lavmodel.new@ceq.function
+                    }
+                    if(is.null(body(lavmodel.new@cin.function))) {
+                         cin <- function(x, ...) { return( numeric(0) ) }
+                    } else {
+                         cin <- lavmodel.new@cin.function
+                    }
+                    CON.JAC <-
+                        rbind(JAC,
+                              numDeriv::jacobian(ceq, x = x.rot),
+                              numDeriv::jacobian(cin, x = x.rot))
+                    attr(CON.JAC, "cin.idx") <- cin.idx + nbord
+                    attr(CON.JAC, "ceq.idx") <- c(1:nbord, ceq.idx + nbord)
+                    attr(CON.JAC, "inactive.idx") <- inactive.idx + nbord
+
+                    lavmodel.new@con.jac <- CON.JAC
+                    lavmodel.new@con.lambda <- c(rep(0, nbord), lambda)
+                }
+
+                # overwrite lavpartable/lavmodel with rotated version
+                lavmodel    <- lavmodel.new
+                lavpartable <- PT.new
+
+                # compute VCOV, taking 'rotation constraints' into account
+                VCOV <- lav_model_vcov(lavmodel        = lavmodel,
+                                       lavsamplestats  = lavsamplestats,
+                                       lavoptions      = lavoptions,
+                                       lavdata         = lavdata,
+                                       lavpartable     = lavpartable,
+                                       lavcache        = lavcache,
+                                       lavimplied      = lavimplied,
+                                       lavh1           = lavh1)
+
+                # compute SE and store them in lavpartable
+                tmp <- lav_model_vcov_se(lavmodel = lavmodel.new,
+                               lavpartable = PT.new,, VCOV = VCOV)
+                lavpartable$se <- tmp
+
+                # store rotated VCOV
+                tmp.attr <- attributes(VCOV)
+                VCOV1 <- VCOV
+                attributes(VCOV1) <- tmp.attr["dim"]
+                lavvcov <- list(se = tmp,
+                                information = lavoptions$information,
+                                vcov = VCOV1)
             }
-        }
-    }
+            if(lavoptions$verbose) {
+                cat(" done.\n")
+            }
+        } # vcov
+    } # efa
 
 
     ####################
@@ -1646,37 +1764,6 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         lavInspect(lavaan, "post.check")
     }
 
-    # FIXME: not scale independent (should use solve(Hessian) %*% g)
-    # but Hessian is not always available (or expensive to compute)
-    #hasExplicitConstraints <- FALSE
-    #if(is.character(constraints) && any(nchar(constraints) > 0L)) {
-    #    hasExplicitConstraints <- TRUE
-    #}
-    #hasNonLinearEqConstraints <- FALSE
-    #if(length(lavmodel@ceq.nonlinear.idx) > 0L) {
-    #    hasNonLinearEqConstraints <- TRUE
-    #}
-    #hasIneqConstraints <- FALSE
-    #if(length(lavmodel@cin.linear.idx) > 0L ||
-    #   length(lavmodel@cin.nonlinear.idx) > 0L) {
-    #    hasIneqConstraints <- TRUE
-    #}
-    #if(!is.null(lavoptions$check.gradient) && lavoptions$check.gradient &&
-    #   lavTech(lavaan, "converged") &&
-    #   !hasExplicitConstraints      &&
-    #   !hasNonLinearEqConstraints   &&
-    #   !hasIneqConstraints) {
-    #    grad <- lavInspect(lavaan, "optim.gradient")
-    #    large.idx <- which(abs(grad) > 0.001)  # better 0.0001?
-    #    if(length(large.idx) > 0L) {
-    #        warning(
-#  "lavaan WARNING: not all elements of the gradient are (near) zero;\n",
-#"                  the optimizer may not have found a local solution;\n",
-#"                  use lavInspect(fit, \"optim.gradient\") to investigate")
-#        }
-#    }
-
-
     lavaan
 }
 
