@@ -215,6 +215,7 @@ sam <- function(model          = NULL,
     dotdotdot.mm$debug <- FALSE
     dotdotdot.mm$verbose <- FALSE
     dotdotdot.mm$check.post <- FALSE # neg lv variances may be overriden
+    dotdotdot.mm$check.gradient <- FALSE # too sensitive in large model (global)
 
     # override with mm.args
     dotdotdot.mm <- modifyList(dotdotdot.mm, mm.args)
@@ -251,6 +252,11 @@ sam <- function(model          = NULL,
 
     for(mm in seq_len(nMMblocks)) {
 
+        if(lavoptions$verbose) {
+            cat("Estimating measurement block ", mm, "[",
+                paste(mm.list[[mm]], collapse = " "), "]\n")
+        }
+
         if(sam.method == "local") {
             # LV.idx.list/OV.idx.list: list per block
             LV.idx.list[[mm]] <- vector("list", nblocks)
@@ -400,7 +406,7 @@ sam <- function(model          = NULL,
 
     if(sam.method == "local") {
         # assemble global LAMBDA/THETA (per block)
-        LAMBDA <- computeLAMBDA(FIT@Model)
+        LAMBDA <- computeLAMBDA(FIT@Model, handle.dummy.lv = FALSE)
         THETA  <- computeTHETA(FIT@Model, fix = FALSE) # keep dummy lv
         if(lavoptions$meanstructure) {
             NU <- computeNU(FIT@Model, lavsamplestats = FIT@SampleStats)
